EXTERNAL_IDENTIFIER_FIELD_NAME = 'externalIdentifier'

MAIN_DATA_FROM_ITEM_SCHEMA = ['title',
                              'id',
                              'category',
                              'description',
                              'detailedDescription',
                              'decision',
                              'externalIdentifier', ]

POD_TEMPLATE_ID_PATTERN = '{0}__format__{1}'
